Package: scTreeViz
Type: Package
Title: R/Bioconductor package to interactively explore and visualize
        single cell RNA-seq datasets with hierarhical annotations
Version: 1.19.0
Authors@R: c( 
          person("Jayaram", "Kancherla", email="jayaram.kancherla@gmail.com", role=c("aut", "cre")),
          person("Hector", "Corrada Bravo", email="hcorrada@gmail.com", role=c("aut")),
          person("Kazi Tasnim", "Zinat", email="kzintas@umd.edu", role=c("aut")),
          person("Stephanie", "Hicks", email="shicks19@jhu.edu", role=c("aut"))
          )
Description: scTreeViz provides classes to support interactive data
        aggregation and visualization of single cell RNA-seq datasets
        with hierarchies for e.g. cell clusters at different
        resolutions. The `TreeIndex` class provides methods to manage
        hierarchy and split the tree at a given resolution or across
        resolutions. The `TreeViz` class extends `SummarizedExperiment`
        and can performs quick aggregations on the count matrix defined
        by clusters.
License: Artistic-2.0
Depends: R (>= 4.0), methods, epivizr, SummarizedExperiment
Imports: data.table, S4Vectors, digest, Matrix, Rtsne, httr, igraph,
        clustree, scran, sys, epivizrData, epivizrServer, ggraph,
        scater, Seurat, SingleCellExperiment, ggplot2, stats, utils
Suggests: knitr, BiocStyle, testthat, SC3, scRNAseq, rmarkdown, msd16s,
        metagenomeSeq, epivizrStandalone, GenomeInfoDb
VignetteBuilder: knitr
biocViews: Visualization, Infrastructure, GUI, SingleCell
Encoding: UTF-8
Collate: 'TreeIndex-class.R' 'TreeIndex-methods.R' 'TreeViz-class.R'
        'TreeViz-methods.R' 'startTreeViz.R' 'TreeVizApp-class.R'
        'EpivizTreeData-class.R' 'helper-functions.R'
        'ClusterHierarchy-class.R'
RoxygenNote: 7.1.1
Config/pak/sysreqs: libcairo2-dev cmake libfontconfig1-dev
        libfreetype6-dev libfribidi-dev libglpk-dev make
        libharfbuzz-dev libbz2-dev libicu-dev libjpeg-dev liblzma-dev
        libpng-dev libtiff-dev libuv1-dev libwebp-dev libxml2-dev
        libssl-dev python3 xz-utils zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-04-28 12:57:06 UTC
RemoteUrl: https://github.com/bioc/scTreeViz
RemoteRef: HEAD
RemoteSha: dd15f611114a451159c57445cab5d61496dbf137
NeedsCompilation: no
Packaged: 2026-05-30 09:55:02 UTC; root
Author: Jayaram Kancherla [aut, cre],
  Hector Corrada Bravo [aut],
  Kazi Tasnim Zinat [aut],
  Stephanie Hicks [aut]
Maintainer: Jayaram Kancherla <jayaram.kancherla@gmail.com>
Built: R 4.6.0; ; 2026-05-30 15:23:10 UTC; unix
