# Generated by roxygen2: do not edit by hand

export(ClusterHierarchy)
export(TreeIndex)
export(TreeViz)
export(aggregateTree)
export(createFromSCE)
export(createFromSeurat)
export(createTreeViz)
export(startTreeviz)
exportClasses(EpivizTreeData)
exportClasses(TreeViz)
exportClasses(TreeVizApp)
exportMethods(aggregateTree)
exportMethods(getNodes)
exportMethods(plot)
exportMethods(show)
exportMethods(splitAt)
import(Rtsne)
import(SingleCellExperiment)
import(SummarizedExperiment)
import(clustree)
import(data.table, except=shift)
import(digest)
import(epivizr)
import(ggraph)
import(httr)
import(igraph)
import(methods)
import(sys)
importClassesFrom(epivizr,EpivizApp)
importClassesFrom(epivizrData,EpivizData)
importClassesFrom(epivizrData,EpivizDataMgr)
importClassesFrom(epivizrData,EpivizMeasurement)
importClassesFrom(epivizrData,SparseEpivizMeasurement)
importClassesFrom(epivizrServer,EpivizServer)
importFrom(Matrix,colSums)
importFrom(Matrix,rowSums)
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,SimpleList)
importFrom(S4Vectors,metadata)
importFrom(Seurat,GetAssayData)
importFrom(Seurat,Reductions)
importFrom(Seurat,as.SingleCellExperiment)
importFrom(data.table,as.data.table)
importFrom(data.table,data.table)
importFrom(data.table,melt)
importFrom(data.table,setnames)
importFrom(data.table,setorder)
importFrom(data.table,setorderv)
importFrom(ggplot2,aes)
importFrom(igraph,as_long_data_frame)
importFrom(igraph,graph_from_data_frame)
importFrom(methods,callNextMethod)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,show)
importFrom(scater,calculateTSNE)
importFrom(scran,getTopHVGs)
importFrom(scran,modelGeneVar)
importFrom(stats,na.omit)
importFrom(utils,str)
importMethodsFrom(epivizrData,register)
