% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TreeViz-class.R
\docType{class}
\name{TreeViz-class}
\alias{TreeViz-class}
\title{TreeViz class wrapper for SummarizedExperiment objects}
\description{
TreeViz class wrapper for SummarizedExperiment objects
}
